package com.schbrain.archetype.initializer.runner;

import cn.hutool.core.io.FileUtil;
import cn.hutool.system.SystemUtil;
import com.schbrain.archetype.initializer.config.properties.ArchetypeProperties;
import com.schbrain.archetype.initializer.maven.MavenUtils;
import com.schbrain.archetype.initializer.param.ArchetypeGenerateParam;
import com.schbrain.common.exception.BaseException;
import com.schbrain.common.util.EnvUtils;
import lombok.extern.slf4j.Slf4j;
import org.eclipse.jgit.api.Git;
import org.eclipse.jgit.api.errors.GitAPIException;
import org.eclipse.jgit.transport.TagOpt;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.ApplicationArguments;
import org.springframework.boot.ApplicationRunner;
import org.springframework.boot.context.properties.EnableConfigurationProperties;
import org.springframework.core.env.Environment;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.stereotype.Component;

import java.io.File;

/**
 * @author liaozan
 * @since 2022/3/20
 */
@Slf4j
@Component
@EnableConfigurationProperties(ArchetypeProperties.class)
public class ArchetypePreparer implements ApplicationRunner {

    @Autowired
    private Environment environment;
    @Autowired
    private ArchetypeProperties archetypeProperties;
    @Autowired
    private ThreadPoolTaskExecutor taskExecutor;

    @Override
    public void run(ApplicationArguments args) {
        taskExecutor.execute(() -> {
            installArchetype();
            warmUpIfRequired();
        });
    }

    public void installArchetype() {
        File archetypeTemplateDirectory = createArchetypeDirectory();
        checkoutArchetype(archetypeTemplateDirectory);
        MavenUtils.install(archetypeTemplateDirectory.getAbsolutePath());
    }

    private void warmUpIfRequired() {
        if (!EnvUtils.runningOnCloudPlatform(environment)) {
            return;
        }
        ArchetypeGenerateParam mockParam = new ArchetypeGenerateParam();
        mockParam.setGroupId("com.schbrain");
        mockParam.setArtifactId("schbrain-init");
        MavenUtils.generate(mockParam);
    }

    private File createArchetypeDirectory() {
        String tempDir = SystemUtil.getUserInfo().getTempDir();
        File archetypeTemplateDirectory = new File(tempDir, "archetype");
        FileUtil.del(archetypeTemplateDirectory);
        return archetypeTemplateDirectory;
    }

    private void checkoutArchetype(File archetypeTemplateDirectory) {
        try (Git ignored = Git.cloneRepository()
                .setTagOption(TagOpt.FETCH_TAGS)
                .setNoCheckout(false)
                .setDirectory(archetypeTemplateDirectory)
                .setURI(archetypeProperties.getSourceUrl())
                .call()) {
            log.info("Success checkout archetype source from {}", archetypeProperties.getSourceUrl());
        } catch (GitAPIException e) {
            throw new BaseException("Failed to checkout archetype source from git repository", e);
        }
    }

}